# -*- coding: utf-8 -*-
# ***********************************************************************
# ******************  CANADIAN ASTRONOMY DATA CENTRE  *******************
# *************  CENTRE CANADIEN DE DONNÉES ASTRONOMIQUES  **************
#
# (c) 2021.                            (c) 2021.
#  Government of Canada                 Gouvernement du Canada
#  National Research Council            Conseil national de recherches
#  Ottawa, Canada, K1A 0R6              Ottawa, Canada, K1A 0R6
#  All rights reserved                  Tous droits réservés
#
#  NRC disclaims any warranties,        Le CNRC dénie toute garantie
#  expressed, implied, or               énoncée, implicite ou légale,
#  statutory, of any kind with          de quelque nature que ce
#  respect to the software,             soit, concernant le logiciel,
#  including without limitation         y compris sans restriction
#  any warranty of merchantability      toute garantie de valeur
#  or fitness for a particular          marchande ou de pertinence
#  purpose. NRC shall not be            pour un usage particulier.
#  liable in any event for any          Le CNRC ne pourra en aucun cas
#  damages, whether direct or           être tenu responsable de tout
#  indirect, special or general,        dommage, direct ou indirect,
#  consequential or incidental,         particulier ou général,
#  arising from the use of the          accessoire ou fortuit, résultant
#  software.  Neither the name          de l'utilisation du logiciel. Ni
#  of the National Research             le nom du Conseil National de
#  Council of Canada nor the            Recherches du Canada ni les noms
#  names of its contributors may        de ses  participants ne peuvent
#  be used to endorse or promote        être utilisés pour approuver ou
#  products derived from this           promouvoir les produits dérivés
#  software without specific prior      de ce logiciel sans autorisation
#  written permission.                  préalable et particulière
#                                       par écrit.
#
#  This file is part of the             Ce fichier fait partie du projet
#  OpenCADC project.                    OpenCADC.
#
#  OpenCADC is free software:           OpenCADC est un logiciel libre ;
#  you can redistribute it and/or       vous pouvez le redistribuer ou le
#  modify it under the terms of         modifier suivant les termes de
#  the GNU Affero General Public        la “GNU Affero General Public
#  License as published by the          License” telle que publiée
#  Free Software Foundation,            par la Free Software Foundation
#  either version 3 of the              : soit la version 3 de cette
#  License, or (at your option)         licence, soit (à votre gré)
#  any later version.                   toute version ultérieure.
#
#  OpenCADC is distributed in the       OpenCADC est distribué
#  hope that it will be useful,         dans l’espoir qu’il vous
#  but WITHOUT ANY WARRANTY;            sera utile, mais SANS AUCUNE
#  without even the implied             GARANTIE : sans même la garantie
#  warranty of MERCHANTABILITY          implicite de COMMERCIALISABILITÉ
#  or FITNESS FOR A PARTICULAR          ni d’ADÉQUATION À UN OBJECTIF
#  PURPOSE.  See the GNU Affero         PARTICULIER. Consultez la Licence
#  General Public License for           Générale Publique GNU Affero
#  more details.                        pour plus de détails.
#
#  You should have received             Vous devriez avoir reçu une
#  a copy of the GNU Affero             copie de la Licence Générale
#  General Public License along         Publique GNU Affero avec
#  with OpenCADC.  If not, see          OpenCADC ; si ce n’est
#  <http://www.gnu.org/licenses/>.      pas le cas, consultez :
#                                       <http://www.gnu.org/licenses/>.
#
# ***********************************************************************

import pytest


from cadcutils.net import GroupProperty, Group, User, Role, Identity


def test_group():
    group_id = 'GrId'
    group = Group(group_id=group_id)

    assert group.group_id == group_id
    assert group.authority == 'ivo://cadc.nrc.ca/gms'
    assert group.uri == 'ivo://cadc.nrc.ca/gms?' + group_id

    authority = 'ivo://authority/gms'
    assert Group(group_id=group_id, authority=authority).authority == authority

    admin = User('0007')
    member = User('008')
    group.add_user_admin(admin)
    group.add_user_member(member)

    assert not group.group_admins
    assert group.user_admins
    assert group.user_admins.pop() == admin
    assert not group.group_members
    assert group.user_members
    assert group.user_members.pop() == member

    admin_gr = Group(group_id='admin')
    member_gr = Group(group_id='member')
    group.group_admins.add(admin_gr)
    group.group_members.add(member_gr)
    assert group.group_admins
    assert group.group_admins.pop() == admin_gr
    assert group.group_members
    assert group.group_members.pop() == member_gr

    with pytest.raises(AttributeError):
        group.add_user_admin('wrong type')
    with pytest.raises(AttributeError):
        group.add_user_member('wrong type')


def test_user():
    internal_id = '0021'
    user = User(internal_id=internal_id)

    assert internal_id == user.internal_id


def test_role():
    assert Role(role_name='owner').get_name() == 'owner'
    assert Role(role_name='admin').get_name() == 'admin'
    assert Role(role_name='member').get_name() == 'member'

    with pytest.raises(ValueError):
        Role('invalid')


def test_group_property():
    group_property = GroupProperty('key', 'value', True)

    assert group_property
    assert group_property.key == 'key'
    assert group_property.value == 'value'
    assert group_property.read_only

    with pytest.raises(ValueError):
        GroupProperty(None, 'value', True)

    with pytest.raises(ValueError):
        GroupProperty('key', None, True)


def test_identity():
    identity = Identity(name='abc', identity_type='HTTP')
    assert identity.name == 'abc'
    assert identity.type == 'HTTP'

    assert Identity(name='abc', identity_type='OpenID') == \
        Identity(name='abc', identity_type='OpenID')

    with pytest.raises(ValueError):
        Identity('', identity_type='HTTP')

    with pytest.raises(ValueError):
        Identity('abc', identity_type='UNKNOWN')
